@@ -113,7 +113,7 @@ func typeOfArtifact(a *latest.Artifact) string {
 
 func timeToListDependencies(ctx context.Context, a *latest.Artifact, cfg docker.Config) (time.Duration, []string, error) {
 	start := time.Now()
-	paths, err := build.DependenciesForArtifact(ctx, a, cfg)
+	paths, err := build.DependenciesForArtifact(ctx, a, cfg, nil)
 	return time.Since(start), paths, err
 }
 
